package  
{
    import flash.ui.Keyboard;
	import flash.events.*;
	import flash.display.SimpleButton;
	import flash.net.registerClassAlias;
	import flash.utils.ByteArray;
	import flash.utils.Dictionary;
	import net.flashpunk.*;
	import net.flashpunk.utils.*;
	import flash.geom.*;
	import rostok.*;
	import game.*;
	import flash.display.StageDisplayState;
	import flash.utils.getQualifiedClassName;
	
	[SWF(width = "1280", height = "480", backgroundColor="0x000000")]

	public class Main extends Engine
	{
		// singleton
		public static var instance:Main;

		public static const GAME_MODE:int = 0;
		public static const DIALOG_MODE:int = 1;
		public static const PAUSE_MODE:int = 2;
		public static const MENU_MODE:int = 3;
		public var mode:int;
		
		public var worlds:Object;
		public var currentWorldName:String;
		public var savedScreenScale:Number;
		public var savedCamPos:Point;
		public var initialScale:Number = 2;
		public var savedFullScreenScale:Number;
		
		private var quickSave:ByteArray;
		
		public function Main() 
		{
			instance = this;
			initialScale = 1;
			super(1280, 480, 30, false);
			FP.screen.scaleX = FP.screen.scaleY = initialScale;
			FP.console.enable();
			FP.console.visible = false;
		}
		
		public function initSerializer():void 
		{
			Serializer.registerClass(Player, "x,y,name,type,visible,active,dx,dy,assetImageName,dotsCollectedTotal,dotsNumberTotal");
			Serializer.registerClass(Dot, "x,y,image,follow");
			Serializer.registerClass(Flower, "x,y,image");
			Serializer.registerClass(Trigger, "name,action,param,x,y,width,height,originX,originY,active,visible");
		}
		
		
		override public function init():void 
		{
			initSerializer();
			stage.addEventListener(Event.RESIZE, stageResizeHandler);
			worlds = new Object();
			worlds["l0"] = new Menu();
			FP.world = worlds["l0"];
			
			Input.define("up", Key.W, Key.UP, Key.SPACE, Key.Z, Key.X);
			Input.define("down", Key.S, Key.DOWN);
			Input.define("left", Key.A, Key.LEFT);
			Input.define("right", Key.D, Key.RIGHT);

			Input.define("toggle_console", 		Key.F1);
			Input.define("toggle_fullscreen", 	Key.F12);
			
			Input.define("save", Key.F8, Key.F5);
			Input.define("load", Key.F9);
		}

		public function loadLevels():void 
		{
			Player.dotsCollectedTotal = 0;
			Player.dotsNumberTotal = 0;
			worlds["l1"] = new Game("level01_03.xml", 20, 90);
			worlds["l2"] = new Game("level02_01.xml", 97, 247);
			worlds["l3"] = new Game("level03_00.xml", 16, 553);
			worlds["l4"] = new Game("level04_00.xml", 20, 234);
			worlds["l5"] = new Game("level05_00.xml");
		}
		
		/**
		 * saves very quickly
		 * @return ByteArray with name of world (string), number of entities, objects properties
		 */
		public function save():ByteArray
		{
			var saveData:ByteArray = new ByteArray();
			// worlds/entities
			var a:Array = new Array();
			var wo:Object = new Object;
			var worldNameList:Array = new Array();
			var worldName:String;
			for (worldName in worlds) 
				if (worlds[worldName] != FP.world) 
					worldNameList.unshift( worldName );
				else
					worldNameList.push( worldName );
			
			for each (worldName in worldNameList) {
				trace("***", worldName, "***");
				a.length = 0;
				worlds[worldName].getAll(a);
				saveData.writeObject(worldName);
				var entitiesNumber:int = 0;
				var entitiesByteArray:ByteArray = new ByteArray();
				for each (var e:Entity in a) 
					if (Serializer.isSupported(e)) {
						entitiesNumber++;
						entitiesByteArray.writeBytes( Serializer.serialize(e) );
						trace("saving", getQualifiedClassName(e), " from ", worldName);
					}
				saveData.writeInt(entitiesNumber);
				saveData.writeBytes(entitiesByteArray);
			}
			// compress
			trace("entitiesNumber:",entitiesNumber, "saveData.lenght",saveData.length);
			saveData.compress();
			trace("saveData.length (compressed)",saveData.length);
			return saveData;
		}
		
		/**
		 * restores quickly
		 */
		public function load(saveData:ByteArray):void 
		{
			if (!saveData) return;
			saveData.uncompress();
			saveData.position = 0;
			// worlds
			var e:Entity;
			var a:Array = new Array();
			var wo:Object = new Object;
			while (saveData.position < saveData.length) {
				var worldName:String = saveData.readObject();
				var entitiesNumber:int = saveData.readInt();
				a.length = 0;
				worlds[worldName].getAll(a);
				// remove all entities of type supported by serializer
				trace("entities in ", worldName, ":", worlds[worldName].count);
				for each (e in a) {
					if (Serializer.isSupported(e)) 
						worlds[worldName].remove(e);
				}
				trace("after removal ", worlds[worldName].count);
				// deserialize all saved entities 
				a.length = 0;
				for (; entitiesNumber > 0;entitiesNumber-- ) {
					e = Serializer.deserialize(saveData);
					a.push(e);
					trace("loading / adding", getQualifiedClassName(e), " to ", worldName);
					//worlds[worldName].add(e);
				}
				// add them all at once, this means that all had load() already called, and are initiated
				// if additiona setting up is necessary it may be placed in added() method
				worlds[worldName].addList(a);
				worlds[worldName].updateLists();
				for each (e in a) if (e.hasOwnProperty("afterLoad")) Object(e)["afterLoad"]();
			}
			FP.world = worlds[worldName];
			// back to normal
			saveData.compress();
		}
		
		public function fitToStage(keepAspect:Boolean = true):void 
		{
			if (!keepAspect) {
				this.scaleX = FP.stage.stageWidth / (FP.width * FP.screen.scaleX);
				this.scaleY = FP.stage.stageHeight / (FP.height * FP.screen.scaleY);
			}
			else {
				var scale:Number;
				if (FP.screen.width / FP.screen.height < FP.stage.stageWidth / FP.stage.stageHeight) {
					scale = FP.stage.stageHeight / (FP.height * FP.screen.scaleY);
					this.x = FP.stage.stageWidth / 2 - (FP.halfWidth * FP.screen.scaleX) * scale;
					this.y = 0;
				}
				else {
					scale = FP.stage.stageWidth / (FP.width * FP.screen.scaleX);
					this.x = 0;
					this.y = FP.stage.stageHeight / 2 - (FP.halfHeight * FP.screen.scaleY) * scale;
				}
				this.scaleX = scale;
				this.scaleY = scale;
			}
			//scale -= scale % 1;
			//scanlines.scaleY = FP.screen.scaleY / 2;
		}
		
		private function stageResizeHandler(e:Event):void
		{
			fitToStage();
		}
		
		public function switchFullScreen(keepAspect:Boolean=true):void
		{
			if (FP.stage.displayState == StageDisplayState.NORMAL) {
				FP.stage.displayState = StageDisplayState.FULL_SCREEN;
			} else {
				FP.stage.displayState = StageDisplayState.NORMAL;
			}
		}
		
		/**
		 * sterowanie przebiegiem gry zdarzenia globalne
		 * 
		 * @param	action string
		 * @param	param string
		 * @param	caller entity issuing this action
		 */ 
		public function performAction(action:String, param:String, caller:Entity = null):void 
		{
			trace(action + " / " + param);
			switch (action) 
			{
				case "changeWorld":		
										if (FP.world == worlds[param]) break;
										FP.world = worlds[param];
										//Game(worlds[param]).resetGame();
										//FP.world = worlds[param] = new Game(Game(worlds[param]).mapName, Game(worlds[param]).px, Game(worlds[param]).py );
										Game(worlds[param]).resetPlayer();
										if (param == "l5") {
											FP.world = Menu.instance;
											Menu.instance.stage = 2;
											if (Player.dotsCollectedTotal < Player.dotsNumberTotal) Menu.instance.stage = 3;
										}
										Assets.getSingleSfx("Powerup.mp3").play();
										Input.clear();
										break;
				case "death":
										Assets.getSingleSfx("Explosion.mp3").play();
										//Game(worlds[param]).resetGame();
										Game(worlds[param]).resetPlayer();
										FP.world = worlds[param];
										Input.clear();
										break;
				case "teleport":
										var x:Number = Number(param.split(";")[0]);
										var y:Number = Number(param.split(";")[1]);
										Player.instance.x = x;
										Player.instance.y = y;
										Player.instance.dx = 0;
										Player.instance.dy = 0;
										Assets.getSingleSfx("Teleport.mp3").play();
										break;
				default:
			}
		}
		override public function update():void 
		{
			if (Input.released("toggle_console"))  { 
				FP.console.visible = !FP.console.visible;
				if (FP.console.visible) Input.mouseCursor = "auto";
				else Input.mouseCursor = "hide";
			}
			if (Input.released("toggle_fullscreen")) switchFullScreen(true);
			
			if (Input.released("save")) {
				quickSave = save();
				Data.writeByteArray("quickSave", quickSave);
				Data.save();
			}
			if (Input.released("load")) {
				Data.load();
				quickSave = Data.readByteArray("quickSave");
				load( quickSave );
			}
			
			super.update();
		}
	}
}