package game
{
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.geom.Vector3D;
	import flash.net.SharedObject;
	import net.flashpunk.*;
	import net.flashpunk.debug.Console;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.utils.*;
	import rostok.*;
	import flash.utils.*;
	
	public class Game extends World
	{
		public static var sharedObject:SharedObject;
		public static var persistentEntities:Array = new Array();
		public var entityToFollow:Entity;
		public var dotsCollected:uint = 0;
		public var dotsNumber:uint = 0;
		public var worldRect:Rectangle = new Rectangle(); // maksymalne wymiary widocznego poziomuy, wyznaczane przez slopes i floors
		public var mapName:String;
		public var px:Number;
		public var py:Number;
		
		public function Game(map:String, px:Number=0, py:Number=0) 
		{
			this.px = px;
			this.py = py;
			resetGame(mapName = map);
		}
		
		override public function begin():void 
		{
			var p:Player = getInstance("player");
			if (p) Player.instance = p;
			if (Player.instance) {
				Player.instance.x = px;
				Player.instance.y = py;
				Player.instance.reassignSprite();
			}
			super.begin();
		}
		
		public function resetGame(map:String=""):void
		{
			dotsCollected= 0;
			dotsNumber= 0;
			worldRect = new Rectangle();
			
			if (map == "") map = mapName;
			removeAll();
			updateLists(false);
			Player.instance = null;
			if (!Player.instance) add(Player.instance = new Player(px, py));
			loadGleedLevel(map);
			FP.screen.color = 0xffffff;	
			Input.mouseCursor = "hide";
		}
		
		public function resetPlayer():void 
		{
			var p:Player = getInstance("player");
			if (p) Player.instance = p;
			Player.instance.x = px;
			Player.instance.y = py;
		}
		/* load glee level
		 */
		public function loadGleedLevel(asset:String, act:Boolean=true, vis:Boolean=true):void 
		{
			var xml:XML = FP.getXML(Assets.getAsset(asset));

			//define some variables that we will use later on
			var e:Entity;
			var o:XML;
			var n:XML;
			var currentLayer:int;
			var tmpStr:String;
			
			currentLayer = 10000;
			
			// objecs layer 
			var assetName:String;
			var assetNumber:int;
			var textureFileName:String;
					
			for each (o in xml..Layer.(
						String(@Name).substr(0, 7) != "lines" && 
						String(@Name).substr(0, 5) != "trash" 
						)..Item) 
			if (o.attributes()[0]=="TextureItem")
			{
				assetName = o.asset_name;
				assetName = assetName.split("\\").pop();
				assetNumber = int(assetName.split("_").pop());
				assetName = assetName.split("_")[0];
				textureFileName = String(o.texture_filename).split("\\").pop();
				var image:Image = Assets.getImageFromSingleBitmapData(textureFileName);
				
				switch (assetName)
				{
					case "player":
								e = add(new Player(o.Position.X, o.Position.Y));
								//Image(e.graphic).darken(1);
								break;
					case "flower1":
					case "flower2":
					case "flower3":
					case "flower4":
					case "flower5":
					case "flower6":
								e = add(new Flower(o.Position.X, o.Position.Y, textureFileName));
								break;
					case "dot":
					case "dot2":
					case "dot3":
					case "dot4":
								e = add(new Dot(o.Position.X, o.Position.Y, textureFileName));
								Player.dotsNumberTotal++;
								//Image(e.graphic).darken(1);
								break;
					default:
								//e = new Entity(o.Position.X, o.Position.Y, Assets.getSingleImage(textureFileName));
								if (!Assets.checkAsset(textureFileName)) continue;
								e = new Entity(o.Position.X, o.Position.Y, Assets.getImageFromSingleBitmapData(textureFileName));
								e.active = false;
								add(e);
								if (e.graphic is Image) {
									image = Image(e.graphic);
									image.scaleX = o.Scale.X;
									image.scaleY = o.Scale.Y;
									worldRect = worldRect.union(new Rectangle(e.x, e.y, image.scaledWidth, image.scaledHeight));
								}
								break;
				}
				if (o.FlipHorizontally == "true") Image(e.graphic).flipped = true;
				if (o.FlipVertically == "true") { 
					Image(e.graphic).scaleY = -1;
					e.graphic.y = Image(e.graphic).height;
				}
				e.layer = currentLayer;
				//if (textureFileName.substring(0, 5) == "trees") Image(e.graphic).darken(1);
			}
			
			currentLayer = 4;
			for each ( var lo:XML in xml..Layer.(String(@Name) == "lines")) 
			{
				for each (o in lo..Item) 
				if (o.attributes()[0]=="PathItem")
				{
					var second:Boolean = false;
					var ax:Number = 0;
					var ay:Number = 0;
					
					for each (var vertex:XML in o..WorldPoints..Vector2) {
						if (second) {
							e = new Line(ax, ay, vertex.X, vertex.Y, FP.getColorRGB(o.LineColor.R, o.LineColor.G, o.LineColor.B), o.LineColor.A / 255.0);
							//e.name = String(o.@Name);
							//e.visible = "True" == String(o.@Visible);
							e.layer = currentLayer;
							add(e);
						}
						ax = vertex.X;
						ay = vertex.Y;
						second = true;
					}
				}
			}
			
			// triggers
			for each ( lo in xml..Layer.(String(@Name).substr(0, 8) == "triggers")) 
			{
				for each (o in lo..Item) 
				{
					e = add(new Trigger(Number(o.Position.X) + Number(o.Width) / 2, Number(o.Position.Y) + Number(o.Height) / 2, o.Width, o.Height, o..Property.(@Name == "action").string, o..Property.(@Name == "param").string));
					e.name = String(o.@Name);
				}
			}
			
			
		}
		
		// przesuwa kamere
		public function setCamera(factor:Number=0.075):void 
		{
			if (!entityToFollow) return;
			// to lekko eliminuje dziwne skakanie elementow o 1 px
			if (Math.abs(entityToFollow.x - FP.halfWidth - camera.x)>1)
				camera.x = int(Utils.lerp( camera.x, entityToFollow.x - FP.halfWidth , factor ));
			if (Math.abs(entityToFollow.y - FP.halfHeight - camera.y)>1) {
				camera.y = int(Utils.lerp( camera.y, entityToFollow.y - FP.halfHeight, factor ));
				camera.y = FP.clamp(camera.y, worldRect.y, worldRect.bottom-FP.screen.height);
			}
		}

		override public function update():void 
		{
			entityToFollow = Player.instance;
			setCamera();
			super.update();
		}

		override public function render():void
		{
			super.render();
			var str:String = "krupki " + Player.dotsCollectedTotal + "/" + Player.dotsNumberTotal;
			FastFont.print(str, FP.camera.x+FP.screen.width-4-FastFont.textWidth(str), FP.camera.y+4, 0x000000);
		}
	}
}