package rostok 
{
	import flash.display.BitmapData;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Text;

	/**
	 * very fast text rendering class
	 * @author rostok
	 */
	public class FastFont 
	{
		[Embed(source = 'BuxtonSketch.ttf', embedAsCFF="false", fontFamily = 'xdefault')] private static var _FONT_XDEFAULT:Class;

		private static var _matrix:Matrix = null;
		private static var _colorTransform:ColorTransform = null;
		private static var _characters:Object = null;
		// adjusted distance between letters
		public static var separator:uint = 4;
		
		public function FastFont() 
		{
			
		}
		
		/**
		 * initializes class (prepares font characters)
		 */
		public static function init():void 
		{
			_matrix = new Matrix();
			_colorTransform = new ColorTransform();
			_characters = new Object();
			for (var i:int = 0; i < 255; i++)
			{
				var s:String = String.fromCharCode(i);
				var txt:Text = new Text( s , 0, 0, { font:"xdefault", size:32, color:0xFFFFFF } ); // change for sth completly different
				_characters[s] = txt;
			}
		}
		
		/**
		 * returns text width
		 * @param	text
		 * @return	returns text width
		 */
		public static function textWidth(text:String):uint
		{
			// remove it and initialize manually for even higher speed
			if (_characters == null) init();
			
			var w:uint = 0;
			for (var i:int = 0; i < text.length; i++) 
			{ 
				var letter:Image = _characters[text.charAt(i)];
				var src:BitmapData = letter.getSource();
				w += letter.width - separator;
			}		
			return w;
		}
		
		/**
		 * draws text 
		 * @param	text	text to be drawn, special chars (newline, CR, tab) are ignored
		 * @param	x		x position
		 * @param	y		y position
		 * @param	color	colour, alpha is omitted
		 * @param	target	destination bitmapdata of FP's screen if null
		 */
		public static function print(text:String, x:int, y:int, color:int=0xFFFFFF, target:BitmapData = null):void 
		{
			// remove it and initialize manually for even higher speed
			if (_characters == null) init();
			
			_colorTransform.redMultiplier = (color >> 16 & 0xFF) / 255.0;
			_colorTransform.greenMultiplier = (color >> 8 & 0xFF) / 255.0;
			_colorTransform.blueMultiplier = (color & 0xFF) / 255.0;
				
			if (!target) target = FP.buffer;
			x -= FP.camera.x;
			y -= FP.camera.y;
			for (var i:int = 0; i < text.length; i++) 
			{ 
				var letter:Image = _characters[text.charAt(i)];
				var src:BitmapData = letter.getSource();
				
				_matrix.tx = x;
				_matrix.ty = y;
				target.draw(src, _matrix, _colorTransform);
				x += src.width - separator;
			}		
		}
	}
}