package rostok {
	import flash.geom.Point;
	import flash.geom.Vector3D;
	
	// plaszczyzna w 2d reprezentowana przez wektor, 
	public class Plane {
		// xy to normalna, w to odleglosc od poczatku ukladu
		public var norm:Vector3D;

		// konstruktor 
		public function Plane()
		{
			norm = new Vector3D();
		}
		
		// z wektora
		public function planeFromVector(n:Vector3D):void
		{
			norm = n;	
		}
			
		// konstruktor z odcinka
		public function planeFromPoints(ax:Number, ay:Number, bx:Number, by:Number):void
		{
			var aa:Vector3D = new Vector3D (ax, ay);
			var ab:Vector3D = new Vector3D (bx - ax, by - ay);
			var bz:Vector3D = new Vector3D (0,0, -1);
			var n:Vector3D;
			
			n = ab.crossProduct(bz);
			n.normalize();
			n.w = n.dotProduct(aa);
			
			norm = n;
			//return Plane(n);
		}
			
		// zmienia kierunek plaszczyzny
		public function invert():void
		{
			norm.x =  -norm.x;	
			norm.y =  -norm.y;	
			norm.w =  -norm.w;	
		}
		
		// odleglosc punktu od tej plaszczyzny
		public function getDistance(px:Number,py:Number):Number
		{
			return norm.x * px + norm.y * py - norm.w;
		}

		// czy punkt jest przed plaszczyzna
		public function isOnFront(px:Number,py:Number):Boolean 
		{
			return getDistance(px, py) > 0;
		}
		
		// czy punkt jest za plaszczyzna
		public function isOnBack(px:Number,py:Number):Boolean 
		{
			return getDistance(px, py) < 0;
		}
		
		// TODO optimize
		/* znajduje przeciecie, zwraca Vector3D 
		 * jesli w<>0 to jest przeciecie
		 * x,y to punkt przeciecia
		 * jesli w==1 to x,y zastepuje drugi punkt
		 * jesli w==-1 to x,y zastepuje pierwszy punkt
		 */
		public function intersect(ax:Number, ay:Number, bx:Number, by:Number):Vector3D
		{
			var vr:Vector3D = new Vector3D(bx - ax, by - ay); // wektor z a do b
			var ad:Number = getDistance(ax,ay);
			var bd:Number = getDistance(bx,by);
			
			// oba po tej samej stronie
			if ((ad < 0 && bd < 0) || (ad > 0 && bd > 0)) return vr;
			
			vr.normalize();
			vr.scaleBy( ad / vr.dotProduct(norm) );
			vr.x = ax-vr.x;
			vr.y = ay-vr.y;

			// jesli a jest z przodu 
			if (ad > 0) 
				vr.w = 1;
			else 
				vr.w = -1;

			return vr;
		}
		
		/* znajduje przeciecie, zwraca Vector3D 
		 * x,y to poczatek odcinka
		 * z,w to koniec
		 * zwrocony odcinek traci to co jest "za" plaszczyzna
		 * jesli nie ma przeciecia zwraca odcinek przecinany
		 */
		public function cutBackSegment(ax:Number, ay:Number, bx:Number, by:Number):Vector3D
		{
			var vr:Vector3D = intersect(ax,ay,bx,by);
			if (vr.w == 0)
			{
				vr.x = ax;
				vr.y = ay;
				vr.z = bx;
				vr.w = by;
			}
			else
			if (vr.w == 1) // a z przodu, wybuj od a do przeciecia
			{
				vr.z = vr.x;
				vr.w = vr.y;
				vr.x = ax;
				vr.y = ay;
			}
			else // a z tylu, wynik od przeciecia do b
			{
				vr.z = vr.x;
				vr.w = vr.y;
				vr.x = bx;
				vr.y = by;
			}
			return vr;
		}
		
		/* znajduje przeciecie, zwraca Vector3D 
		 * x,y to poczatek odcinka
		 * z,w to koniec
		 * zwrocony odcinek traci to co jest "przed" plaszczyzna
		 * jesli nie ma przeciecia zwraca odcinek przecinany
		 */
		public function cutFrontSegment(ax:Number, ay:Number, bx:Number, by:Number):Vector3D
		{
			var vr:Vector3D = intersect(ax,ay,bx,by);
			if (vr.w == 0)
			{
				vr.x = ax;
				vr.y = ay;
				vr.z = bx;
				vr.w = by;
			}
			else
			if (vr.w == -1) // a z tylu, wynik odcinek od a do przeciecia
			{
				vr.z = vr.x;
				vr.w = vr.y;
				vr.x = ax;
				vr.y = ay;
			}
			else // a z przodu, wynik od przeciecia do b
			{
				vr.z = vr.x;
				vr.w = vr.y;
				vr.x = bx;
				vr.y = by;
			}
			return vr;
		}

		public function toString():String
		{
			return norm.x + " " + norm.y + " " + norm.w;
		}
	}
}