package rostok 
{
	import flash.utils.ByteArray;
	import flash.utils.getDefinitionByName;
	import flash.utils.getQualifiedClassName;
	
	/**
	 * class to serialize and deserialize objects in ByteArray
	 * @author rostok
	 */
	public class Serializer 
	{
		// an object of arrays, key is object's class name, value is array of property names
		private static var objectsProperties:Object = new Object();
		
		public function Serializer() 
		{
		}
		
		/** 
		 * reisters class and selected properties
		 * @cl	a class to save properties for
		 * @propname	is a name of property or string of names separated by commas
		 */ 
		public static function registerClass(cl:Class, propname:String=""):void 
		{
			var cn:String = getQualifiedClassName(cl);
			var pn:Array;
			if (objectsProperties[cn]) 
				pn = objectsProperties[cn];
			else	
				pn = objectsProperties[cn] = new Array();
			
			var a:Array = propname.split(",");
			for each (var pp:String in a) {
				pp = pp.replace(" ", ""); // remove spaces
				if (pn.indexOf(pp)<0)
					pn.push(pp);
			}
		}
		
		/**
		 * returns true if this class was added to Serializer
		 */
		public static function isSupported(o:*):Boolean
		{
			return objectsProperties[ getQualifiedClassName(o) ] != null;
		}
		
		/**
		 * saves properties to ByteArray
		 * only properties that were registered earlier are saved
		 * if object's parent class was registered parent't properties are saved as well
		 * if object has save() method it is called before writing to stream
		 * @src	any object
		 * @return	new ByteArray
		 */ 
		public static function serialize(src:*):ByteArray
		{
			if (src.hasOwnProperty("save")) src.save();
			var o:Object = new Object();
			o["___className___"] = getQualifiedClassName(src);

			var defined:Boolean = false;
			for (var cn:String in objectsProperties) 
				if (src is Class(getDefinitionByName(cn)))
				{
					defined = true;
					//trace("serializing ", src, " as ", cn);
					var pn:Array = objectsProperties[cn];
					for each (var p:String in pn) {
						if (src.hasOwnProperty(p)) o[p] = Object(src)[p];
					}
				}
			if (!defined) throw("Serializer:serialize() object not defined");

			var ba:ByteArray = new ByteArray();
			ba.writeObject(o);
			return ba;
		}
		
		/**
		 * reads object from ByteArray's current position
		 * creates it and sets its properties
		 * if object has load() method it is called after all properties were set
		 */ 
		public static function deserialize(ba:ByteArray):*
		{
			var o:Object = ba.readObject();
			//trace("deserialize " + o["___className___"]);
			var temp:*;
			
			temp = new (getDefinitionByName(o["___className___"]));
			//if (o["___className___"] == "game::CatBearRabbit") trace("loading CBR x/y:", o["x"], o["y"]);
			
			for(var p:String in o) {
				if (p != "___className___")	
					temp[p] = o[p];
			}
			if (temp.hasOwnProperty("load")) temp.load();
			return temp;
		}
	}
}