package rostok  {
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.geom.Vector3D;
	import rostok.Plane;
	import rostok.Utils;
	import net.flashpunk.utils.Draw;
	import net.flashpunk.Entity;
	
	/**
	 * klasa reprezentujaca zamkniety obszar wypukly w 2d
	 * jest to wieloscian wypukly reprezentowany przez tablice kolejnych punktow
	 * oraz tablice plaszczyzn wyznaczonych przez odcinki
	 */
	public class Solid {
		public var xpts:Vector.<Number>;
		public var ypts:Vector.<Number>;
		public var planes:Vector.<Plane>;
		public var boundingRect:Rectangle = null;
//		public var midX:Number;
//		public var midY:Number;
		
		// konstruktor
		public function Solid()
		{
			xpts = new Vector.<Number>();
			ypts = new Vector.<Number>();
			planes = new Vector.<Plane>();
		}
		
		// klonuje solid
		public function clone():Solid
		{
			var s:Solid = new Solid();
			for (var i:uint = 0; i < xpts.length; i++) {
				s.addVertex(xpts[i], ypts[i]);
			}
			return s;
		}
		
		/** 
		 * czysci solid, zeruje dlugosci tablic
		 */
		public function clear():void 
		{
			xpts.length = 0;
			ypts.length = 0;
			planes.length = 0;
		}
		
		/**
		 * zamienia miejscami dwa punty w solidzie
		 * @param	a indeksy punktow
		 * @param	b
		 */
		public function swapPoints(a:uint, b:uint):void
		{
			var n:Number; 
			n = xpts[a];
			xpts[a] = xpts[b];
			xpts[b] = n;

			n = ypts[a];
			ypts[a] = ypts[b];
			ypts[b] = n;
			
			recalcPlane(a);
			recalcPlane(b);
		}
		
		/**
		 * tworzy ponownie plaszczyzne dla punktu o indeksie i, takze nastepna
		 */
		public function recalcPlane(i:uint):void
		{
			if (i >= xpts.length) return;
			
			var nextIndex:uint = i + 1;
			if (nextIndex == xpts.length) nextIndex = 0;
			var previousIndex:int = i - 1;
			if (previousIndex == -1) previousIndex = xpts.length-1;
			
			Plane(planes[i]).planeFromPoints( xpts[i], ypts[i], xpts[nextIndex], ypts[nextIndex] );
			Plane(planes[previousIndex]).planeFromPoints( xpts[previousIndex], ypts[previousIndex], xpts[i], ypts[i] );
		}

		/**
		 * oblicza srodek geometryczny solida
		 */
		/*
		public function calcMidPoint():void 
		{
			if (xpts.length == 0) return;
			midX = midY = 0;
			for (var i:int = 0; i < xpts.length; i++)
			{
				midX += xpts[i];
				midY += ypts[i];
			}
			midX /= xpts.length;
			midY /= xpts.length;
		}
		*/
		
		/**
		 * dodaje punkt do solida,
		 * mozna dodawac vertexy counterclockwise
		 */
		public function addVertex(x:Number, y:Number):void
		{
			var p:Plane = new Plane();
			
			if (xpts.length == 0)
			{
				//xpts[xpts.length]=x;
				//ypts[ypts.length]=y;
				//planes[planes.length]=p;
				xpts.push(x);
				ypts.push(y);
				planes.push( p );
				return;
			} 
			if (xpts.length > 0)
			{
				if ( xpts.length > 1 && Plane(planes[xpts.length - 1]).isOnBack( x, y ) )
				{
					if (xpts.length == 2)
					{
						swapPoints(0, 1);
					}
					else
					{
						return;
					}
				}
				else
				if (xpts.length > 2)
				{
					if ( Plane(planes[xpts.length - 2]).isOnFront( x, y ) ) return;
					if ( Plane(planes[0]).isOnFront( x, y ) ) return;
				}
				//xpts[xpts.length]=x;
				//ypts[ypts.length]=y;
				xpts.push(x);
				ypts.push(y);
				// ostatnia plaszczyzna laczy nowy punkt z pierwszym
				p.planeFromPoints( x, y, xpts[0], ypts[0] );
				//planes[planes.length]=p;
				planes.push( p );

				// przedostatnia plaszczyzna laczy poprzedni z nowym
				Plane(planes[xpts.length-2]).planeFromPoints( xpts[xpts.length-2], ypts[xpts.length-2], x, y );
			}
			//calcMidPoint();
		}
		
		/**
		 * oblicza prostokat okalajacy solida
		 */
		public function recalcBoundingRect():void 
		{
			if (xpts.length == 0) return;
			boundingRect = new Rectangle(xpts[0], ypts[0], 0, 0);
			for (var i:uint = 1; i < xpts.length; i++)
			{
				boundingRect.x = Math.min(boundingRect.x, xpts[i]);
				boundingRect.y = Math.min(boundingRect.y, ypts[i]);
				boundingRect.right = Math.max(boundingRect.x, xpts[i]);
				boundingRect.bottom = Math.max(boundingRect.y, ypts[i]);
			}
		}
		
		/** 
		 * przelicza wszystkie plaszczyzny
		 */
		public function recalcAllPlanes():void 
		{
			for (var i:uint = 0; i < planes.length; i++) 
				recalcPlane(i);			
			//calcMidPoint();
			recalcBoundingRect();
		}
		
		/** 
		 * przeskalowuje punkty solida tak, aby miescil sie we wskazanym prostokacie
		 * rowniez przelicza wszystkie plaszczyzny
		 */
		public function fitToRect(x1:Number, y1:Number, x2:Number, y2:Number):void 
		{
			if (xpts.length < 1) return;
			var _x1:Number = xpts[0];
			var _y1:Number = ypts[0];
			var _x2:Number = xpts[0];
			var _y2:Number = ypts[0];
			var i:uint;
			for (i = 1; i < xpts.length; i++)
			{
				_x1 = Math.min(_x1, xpts[i]);
				_x2 = Math.max(_x2, xpts[i]);
				_y1 = Math.min(_y1, ypts[i]);
				_y2 = Math.max(_y2, ypts[i]);
			}
			for (i = 0; i < xpts.length; i++)
			{
				xpts[i] = Utils.lerp(x1, x2, (xpts[i] - _x1) / (_x2 - _x1));
				ypts[i] = Utils.lerp(y1, y2, (ypts[i] - _y1) / (_y2 - _y1));
			}
			recalcAllPlanes();
		}

		/** 
		 * wpasowywuje solid w hitbox 
		 */
		public function fitToHitbox(e:Entity):void
		{
			fitToRect( e.x - e.originX,  			e.y - e.originY, 
				       e.x - e.originX + e.width, 	e.y - e.originY + e.height);
		}
		
		/**
		 * znajduje przeciecie, zwraca w output Vector3D 
		 * @x
		 * @y to poczatek odcinka
		 * @z
		 * @w to koniec
		 * @output zwrocony odcinek traci to co jest "w" solidzie, czyli "za" jego wszystkimi plaszczyznami, jesli nie ma przeciecia zwraca argument
		 * 
		 * @returns true jesli bylo ciecie, false jesli nie bylo
		 */
		public function cutBack(p1x:Number, p1y:Number, p2x:Number, p2y:Number, output:Vector3D):Boolean
		{
			var wasCut:Boolean = false;
			output.x = p1x;
			output.y = p1y;
			output.z = p2x;
			output.w = p2y;
			if (xpts.length < 2) return false;
			
			var xpts_length:uint = xpts.length-1;
			for (var i:uint = 0; i < xpts_length; i++)
				wasCut != Utils.cutSegment(xpts[i],ypts[i],xpts[i+1],ypts[i+1], output.x, output.y, output.z, output.w, output);
			
			wasCut != Utils.cutSegment(xpts[i],ypts[i],xpts[0],ypts[0], output.x, output.y, output.z, output.w, output);
			
			return wasCut;
		}
		
		/**
		 * sprawdza czy odcinek przecina solid
		 */
		public function isSegmentIntersection(p1x:Number, p1y:Number, p2x:Number, p2y:Number):Boolean
		{
			if (xpts.length < 2) return false;
			if (p1x == p2x && p1y == p2y) return false;
			var i:uint;
			
/*			// sprawdzamy czy kazdy odcinek lezy po jednej stronie prostej
			var nx:Number = p2y - p1y;
			var ny:Number = - (p2x - p1x);
			var nl:Number = Math.sqrt(nx * nx + ny * ny);
			nx /= nl;
			ny /= nl;
			var nu:Number = nx * p1x + ny * p1y;
			var b:Boolean = true;
			// nx ny nu to normalna prostej p1p2 jesli punkty solida leza po jednej stronie to przeciecia nie ma
			if (nx * xpts[0] + ny * ypts[0] < nu) 
			{
				for (i = 1; b && i < xpts.length - 1; i++) if (nx * xpts[i] + ny * ypts[i] >= nu) b = false;
			}
			else 
			{
				for (i = 1; b && i < xpts.length - 1; i++) if (nx * xpts[i] + ny * ypts[i] < nu) b = false; 
			}
			if (b) return false;
*/

			// przeciecia segmentu z kazdym bokiem
			var xpts_length:uint = xpts.length-1;
			for (i = 0; i < xpts_length; i++)
				if ( Utils.ptsits2d(xpts[i], ypts[i], xpts[i + 1], ypts[i + 1], p1x, p1y, p2x, p2y) ) return true;
			
			if ( Utils.ptsits2d(xpts[i], ypts[i], xpts[0], ypts[0], p1x, p1y, p2x, p2y) ) return true;
			
			return false;
		}
		
		/**
		 * sprawdza czy odcinek przecina kapsule o promieniu r
		 */
		public function isCapsuleIntersection(p1x:Number, p1y:Number, p2x:Number, p2y:Number,r:Number=0):Boolean
		{
			if (xpts.length < 2) return false;
			if (p1x == p2x && p1y == p2y) return false;
			var i:uint;
			
			// przeciecia segmentu z kazdym bokiem
			var xpts_length:uint = xpts.length-1;
			for (i = 0; i < xpts_length; i++)
				if ( Utils.capsulesIntersect(xpts[i], ypts[i], xpts[i + 1], ypts[i + 1], p1x, p1y, p2x, p2y, r) ) return true;
			
			if ( Utils.capsulesIntersect(xpts[i], ypts[i], xpts[0], ypts[0], p1x, p1y, p2x, p2y, r) ) return true;
			
			return false;
		}
		
		/** 
		 * mowi czy punkt jest wewnatrz solida
		 * jesli ktorykolwiek jest z przodu to nie jest
		 * punkt musi byc za wszystkimi plaszczyznami
		 * 
		 * jesli jest ustawiony boudingRect to przyspiesza sprawdzanie przez wykrycie czy x/y jest w bouding boxie
		 */
		public function isInside(x:Number, y:Number, fastCheck:Boolean=true):Boolean
		{
			if (fastCheck && boundingRect && !boundingRect.contains(x, y)) return false;
			var planes_length:uint = planes.length;
			for (var i:uint = 0; i < planes_length; i++)
				if (Plane(planes[i]).getDistance(x, y) > 0) return false;
			return true;
		}
		
		/** 
		 * mowi czy okrag przecina solid
		 * jesli ktorykolwiek jest z przodu to nie jest
		 * punkt musi byc za wszystkimi plaszczyznami
		 */
		public function isInsideRadius(x:Number, y:Number, r:Number=0):Boolean
		{
			var planes_length:uint = planes.length;
			for (var i:uint = 0; i < planes_length; i++)
				if (Plane(planes[i]).getDistance(x, y) > r) return false;
			return true;
		}
		
		/** 
		 * rysuje dany solid
		 * ta wersja rysuje polodcinki, gdzie pusta czesc oznacza punkt poczatkowy
		 */
		public function draw(color:int = 0xFFFFFF):void
		{
			
			if (xpts.length>1) Draw.line(xpts[xpts.length-1], ypts[xpts.length-1], xpts[0], ypts[0], color);
			var xpts_length:uint = xpts.length;
			for (var i:uint = 1; i < xpts_length; i++)
			{
				Draw.line(xpts[i - 1], ypts[i - 1], xpts[i], ypts[i], color);
			}
			//Draw.line(midX,midY,midX,midY, color);
			return;
		/*	if (xpts.length>1) Draw.line((xpts[xpts.length-1]+xpts[0])/2, (ypts[xpts.length-1]+ypts[0])/2, xpts[0], ypts[0], color);
			for (var i:int = 1; i < xpts.length; i++)
			{
				Draw.line((xpts[i - 1]+xpts[i])/2, (ypts[i - 1]+ypts[i])/2, xpts[i], ypts[i], color);
			}
*/
		}
		
		/** 
		 * przesuwa solid i przelicza plaszcz
		 */
		public function translate(dx:Number, dy:Number, recalc:Boolean = true):void
		{
			var xpts_length:uint = xpts.length;
			for (var i:uint = 0; i < xpts_length; i++)
			{
				xpts[i] += dx;
				ypts[i] += dy;
			}
			if (recalc) recalcAllPlanes();
		}
		
		/** 
		 * skaluje solid wzgledem x,y
		 */
		public function scale(fraction:Number, x:Number = 0, y:Number=0 ):void 
		{
			translate( -x, -y);
			var xpts_length:uint = xpts.length;
			for (var i:uint = 0; i < xpts_length; i++)
			{
				xpts[i] *= fraction;
				ypts[i] *= fraction;
			}
			translate( x, y, true);
		}
		/** 
		 * obraca solid wzgledem srodka ukladu
		 */
		public function rotate(angleDegrees:Number, recalc:Boolean = true):void
		{
			var p:Point = new Point();
			var m:Matrix = new Matrix();
			m.rotate(angleDegrees/180*Math.PI);
			var xpts_length:uint = xpts.length;
			for (var i:uint = 0; i < xpts_length; i++)
			{
				p.x = xpts[i];
				p.y = ypts[i];
				p = m.transformPoint(p);
				xpts[i] = p.x;
				ypts[i] = p.y;
			}
			if (recalc) recalcAllPlanes();
		}

		/** 
		 * obraca solid wzgledem pierwszego punktu
		 */
		public function rotateFirst(angleDegrees:Number):void
		{
			if (xpts.length < 2) return;
			var p:Point = new Point(xpts[0], ypts[0]);
			translate( -p.x, -p.y, false);
			rotate(angleDegrees, false);
			translate( p.x, p.y);
		}
		
		/** 
		 * przesuwa kazdy z punktow w kierunku nastepnego o ulamek calej drogi
		 */
		public function translateVertsForward(fraction:Number):void
		{
			if (xpts.length < 3) return;
			var p:Point = new Point(xpts[0],ypts[0]);
			var xpts_length:uint = xpts.length;
			for (var i:uint = 1; i < xpts_length; i++)
			{
				xpts[i - 1] = Utils.lerp(xpts[i - 1], xpts[i], fraction);
				ypts[i - 1] = Utils.lerp(ypts[i - 1], ypts[i], fraction);
			}
			xpts[i - 1] = Utils.lerp(xpts[i - 1], p.x, fraction);
			ypts[i - 1] = Utils.lerp(ypts[i - 1], p.y, fraction);
			recalcAllPlanes();
		}
		
		/**
		 * tworzy rect z hitboxa entity 
		 */
		public function createRectFromHitbox(e:Entity):void
		{
			clear();
			addVertex(e.x - e.originX, e.y - e.originY);
			addVertex(e.x - e.originX, e.y - e.originY + e.height);
			addVertex(e.x - e.originX + e.width, e.y - e.originY + e.height);
			addVertex(e.x - e.originX + e.width, e.y - e.originY);
		}
		
		/**
		 * slizga sie po powierzchni solida, dobre do rzutowania wektora ruchu. zwraca rzutowany w output
		 * 
		 * @returns true jesli bylo rzutowanie, false jesli nie bylo przeciecia
		 */
		public function cutSegmentAndSlide(p1x:Number, p1y:Number, p2x:Number, p2y:Number, output:Vector3D):Boolean
		{
			// trzeba wykryc ostatni odcinek, ktory przetnie segment i dla niego zrobic slide
			output.x = p1x;
			output.y = p1y;
			output.z = p2x;
			output.w = p2y;

			if (xpts.length < 2) return false;
			var lastcutter:int = -2;
			var xpts_length:uint = xpts.length;
			for (var i:uint = 0; i < xpts_length - 1; i++) {
				if (Utils.cutSegment(xpts[i], ypts[i], xpts[i + 1], ypts[i + 1], output.x, output.y, output.z, output.w, output)) lastcutter = i;
			}
			
			// jesli bylo ciecie wiec odcinek 'i' cial go	
			if (Utils.cutSegment(xpts[i],ypts[i],xpts[0],ypts[0], output.x, output.y, output.z, output.w, output)) lastcutter = -1;
			
			// nie bylo ciecia, newseg teraz to ostatni ciety
			if (lastcutter == -2) return false; 
			
			// ciecie ale ostatni-pierwszy, newseg staje cie tnacym
			//cut and slide	
			if (lastcutter == -1) 
				Utils.slideOnSegment(xpts[i], ypts[i], xpts[0], ypts[0], p1x, p1y, p2x, p2y, output);
			else
				Utils.slideOnSegment(xpts[lastcutter], ypts[lastcutter], xpts[lastcutter+1], ypts[lastcutter+1], p1x, p1y, p2x, p2y, output);

			
			return true; 
		}
		
		/**
		 * odbija segment od solida,
		 * zwraca odbity
		 */
		public function reflectSegment(p1x:Number, p1y:Number, p2x:Number, p2y:Number, output:Vector3D):Boolean
		{
			// trzeba wykryc ostatni odcinek, ktory przetnie segment i dla niego zrobic slide
			output.x = p1x;
			output.y = p1y;
			output.z = p2x;
			output.w = p2y;
			if (xpts.length < 2) return false;
			var lastcutter:int = -2;
			var xpts_length:uint = xpts.length;
			for (var i:uint = 0; i < xpts_length - 1; i++) {
				if (Utils.cutSegment(xpts[i], ypts[i], xpts[i + 1], ypts[i + 1], output.x, output.y, output.z, output.w, output)) 
					lastcutter = i;
			}
			
			// jesli bylo ciecie wiec odcinek 'i' cial go	
			if (Utils.cutSegment(xpts[i], ypts[i], xpts[0], ypts[0], output.x, output.y, output.z, output.w, output))
				lastcutter = -1;
			
			// nie bylo ciecia, newseg teraz to ostatni ciety
			if (lastcutter == -2) return false; 
			
			// ciecie ale ostatni-pierwszy, newseg staje cie tnacym
			//cut and reflect
			if (lastcutter == -1) 
				Utils.reflectOnSegment(xpts[i], ypts[i], xpts[0], ypts[0], p1x, p1y, p2x, p2y, output);
			else
				Utils.reflectOnSegment(xpts[lastcutter], ypts[lastcutter], xpts[lastcutter+1], ypts[lastcutter+1], p1x, p1y, p2x, p2y, output);

			
			return true; 
		}

		/**
		 * robi prostokat
		 */
		public function rect(x1:Number, y1:Number, x2:Number, y2:Number):void 
		{
			clear();
			addVertex(Math.min(x1,x2), Math.min(y1,y2));
			addVertex(Math.min(x1,x2), Math.max(y2,y1));
			addVertex(Math.max(x2,x1), Math.max(y2,y1));
			addVertex(Math.max(x2,x1), Math.min(y1,y2));
		}
		
		/**
		 * robi wielokat formemny
		 * pierwszy argument to promien
		 * drugi to liczba punktow
		 * ostani kat pomiedzy osia pozioma a pierwszym punktem 
		 * (w szczegolnosci jestli ten kat to 45 stopni to wyjdzie kwadrat, dla 0 stopni wyjdzie romb)
		 */
		public function regularPolygon(radius:Number, vertNum:uint, startAngle:Number=0):void 
		{
			if (vertNum < 3) return;
			clear();
			for (var i:uint = 0; i < vertNum; i++)
			{
				addVertex(radius,0);
				rotate(360.0 / (vertNum), false);
			}
			rotate(startAngle);
		}
	}
}