package rostok 
{
	import flash.geom.Vector3D;
	import net.flashpunk.FP;
	import net.flashpunk.Graphic;
	import net.flashpunk.utils.Draw;
	
	/**
	 * a simple catmull-rom spline implementation
	 * a spline that passes through all control points
	 * 
	 * @author rostok
	 */
	public class Spline 
	{
		
		/* sets ouput to an interpolated spline value
		 * p 0..1
		 * points vector of vectors
		 */ 
		public static function splineCatmullRom(p:Number, points:Vector.<Vector3D>, output:Vector3D):void
		{
			if (points.length < 2) return;
			p = FP.clamp(p, 0, 1);
			//p = ((p < 0 ? 0 : p) > 1 ? 1 : p);
			
			var plm1:int = points.length - 1;
			var i:int = (plm1) * p;
            var p0:Vector3D = points [FP.clamp(i-1,0,points.length-1)];
            var p1:Vector3D = points [i];
            var p2:Vector3D = points [FP.clamp(i+1,0,points.length-1)];
            var p3:Vector3D = points [FP.clamp(i+2,0,points.length-1)];
            //var p0:Vector3D = points [i-1<0?0:i-1];
            //var p1:Vector3D = points [i<0?0:i];
            //var p2:Vector3D = points [i+1>plm1 ? plm1 : i+1];
            //var p3:Vector3D = points [i+2>plm1 ? plm1 : i+2];
			p *= plm1;
			p -= i;
			//spline(p0, p1, p2, p3, p, output);
			output.x = 0.5 * ((2 * p1.x) + p * ((-p0.x + p2.x) + p * ((2 * p0.x - 5 * p1.x + 4 * p2.x - p3.x) + p * (-p0.x + 3 * p1.x - 3 * p2.x + p3.x))));
			output.y = 0.5 * ((2 * p1.y) + p * ((-p0.y + p2.y) + p * ((2 * p0.y - 5 * p1.y + 4 * p2.y - p3.y) + p * (-p0.y + 3 * p1.y - 3 * p2.y + p3.y))));
			output.z = 0.5 * ((2 * p1.z) + p * ((-p0.z + p2.z) + p * ((2 * p0.z - 5 * p1.z + 4 * p2.z - p3.z) + p * (-p0.z + 3 * p1.z - 3 * p2.z + p3.z))));
		}
		
		public static function spline(p0:Vector3D, p1:Vector3D, p2:Vector3D, p3:Vector3D, t:Number, out:Vector3D):void
		{
			out.x = 0.5 * ((2 * p1.x) + t * ((-p0.x + p2.x) + t * ((2 * p0.x - 5 * p1.x + 4 * p2.x - p3.x) + t * (-p0.x + 3 * p1.x - 3 * p2.x + p3.x))));
			out.y = 0.5 * ((2 * p1.y) + t * ((-p0.y + p2.y) + t * ((2 * p0.y - 5 * p1.y + 4 * p2.y - p3.y) + t * (-p0.y + 3 * p1.y - 3 * p2.y + p3.y))));
			out.z = 0.5 * ((2 * p1.z) + t * ((-p0.z + p2.z) + t * ((2 * p0.z - 5 * p1.z + 4 * p2.z - p3.z) + t * (-p0.z + 3 * p1.z - 3 * p2.z + p3.z))));
		}
		
		public static function drawSpline(points:Vector.<Vector3D>, steps:Number=16):void 
		{
			var v0:Vector3D = new Vector3D;
			var v1:Vector3D = new Vector3D;
			for (var i:int = 0; i < steps; i++)
			{
				splineCatmullRom(i * 1.0 / steps, points, v0);
				splineCatmullRom((i+1) * 1.0 / steps, points, v1);
				Draw.line(v0.x, v0.y, v1.x, v1.y);
			}
		}
		
		public static function drawSpline3d(points:Vector.<Vector3D>, steps:Number=16, color:int=0xFFFFFF, shiftX:Number=0, shiftY:Number=0):void 
		{
			var v0:Vector3D = new Vector3D;
			var v1:Vector3D = new Vector3D;
			for (var i:int = 0; i < steps; i++)
			{
				splineCatmullRom(i * 1.0 / steps, points, v0);
				splineCatmullRom((i+1) * 1.0 / steps, points, v1);
				Draw.line(shiftX+v0.x, shiftY+v0.y-v0.z, shiftX+v1.x, shiftY+v1.y-v1.z,color,1);
			}
		}
	}

}